import requests


def download_file(url, filename):
    r = requests.get(url, stream=True)
    with open(filename, 'wb') as fd:
        for chunk in r.iter_content(chunk_size=128):
            fd.write(chunk)


def make_unique(original_list):
    unique_list = []
    [unique_list.append(obj) for obj in original_list if obj not in unique_list]
    return unique_list



# coding: utf-8
from django.conf import settings
from django.contrib import (admin, messages)
from django_object_actions import DjangoObjectActions
from django.utils.translation import ugettext as _

from . import api
from . import models


class TildaPageAdmin(DjangoObjectActions, admin.ModelAdmin):
    list_display = ('title', 'id', 'synchronized', 'created', )
    list_filter = ('synchronized', 'created', )
    search_fields = ('title', 'id', )
    readonly_fields = (
        'id',
        'title',
        'html',
        'images',
        'css',
        'js',
        'synchronized',
        'created',
    )

    def has_add_permission(self, request):
        return False

    def fetch_pages(modeladmin, request, queryset):
        if api.api_getpageslist():
            messages.add_message(
                request,
                messages.SUCCESS,
                _(u'Pages successfuly fetched from Tilda')
            )
        else:
            messages.add_message(
                request,
                messages.ERROR,
                _(u'Nothing fetched. Perharps wrong settings')
            )
    fetch_pages.label = _(u'Fetch pages')

    def synchronize_page(self, request, obj):
        if api.api_getpageexport(obj.id):
            messages.add_message(
                request,
                messages.SUCCESS,
                _(u'Page «{}» successfuly synced from Tilda'.format(obj.title))
            )
        else:
            messages.add_message(
                request,
                messages.ERROR,
                _(u'Something wrong...')
            )
    synchronize_page.label = _(u'Synchronize')

    change_actions = ('synchronize_page', )
    changelist_actions = ('fetch_pages', )

admin.site.register(models.TildaPage, TildaPageAdmin)



from django import forms
from django.conf import settings
from django.db import models
from django.template.loader import render_to_string
from django.apps import apps


class TildaWidget(forms.Widget):

    def render(self, name, value, attrs=None):
        is_required = self.is_required

        if not hasattr(settings, 'TILDA_PUBLIC_KEY') or \
           not hasattr(settings, 'TILDA_SECRET_KEY') or \
           not hasattr(settings, 'TILDA_PROJECTID') or \
           not settings.TILDA_PUBLIC_KEY or \
           not settings.TILDA_SECRET_KEY or \
           not settings.TILDA_PROJECTID:
            is_need_config = True

        TildaPage = apps.get_model('tilda', 'TildaPage')
        queryset = TildaPage.objects.all()

        if queryset and value:
            obj = queryset.filter(id=value)[0]
        return render_to_string('tilda/widget.html', locals())


class TildaPageField(models.ForeignKey):

    def __init__(self, *args, **kwargs):
        kwargs['to'] = 'tilda.TildaPage'
        super(TildaPageField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs['widget'] = TildaWidget
        return super(TildaPageField, self).formfield(**kwargs)



import os
import requests
from django.conf import settings
from django.utils.timezone import now

from .helpers import download_file, make_unique
from . import models


API_HOST = 'http://api.tildacdn.info/v1'
API_PAYLOAD = {
    'publickey': settings.TILDA_PUBLIC_KEY,
    'secretkey': settings.TILDA_SECRET_KEY
}


def api_getpageslist():
    url = '{}/getpageslist'.format(API_HOST)
    payload = API_PAYLOAD.copy()
    payload['projectid'] = settings.TILDA_PROJECTID
    req = requests.get(url, params=payload)
    if req.status_code == 200:
        res = req.json()
        if res['status'] == 'FOUND':
            for r in res['result']:
                obj, created = models.TildaPage.objects.get_or_create(
                    id=r['id']
                )

                obj.title = r['title']
                obj.save()
            return True
    return False


def api_getpageexport(page_id):
    url = '{}/getpageexport'.format(API_HOST)
    page = models.TildaPage.objects.get(id=page_id)
    payload = API_PAYLOAD.copy()
    payload['pageid'] = page.id
    req = requests.get(url, params=payload)
    if req.status_code == 200:
        res = req.json()
        if res['status'] == 'FOUND':

            """
                Remove old images
            """

            for img in page._path_images_list():
                if os.path.exists(img):
                    os.remove(img)

            """
                Download new images, css, js
            """

            result = res['result']
            page.title = result['title']
            page.html = result['html']
            page.images = result['images']
            page.css = result['css']
            page.js = result['js']
            page.synchronized = now()
            page.save()

            for r in make_unique(result['images']):
                filename = os.path.join(settings.TILDA_MEDIA_IMAGES, r['to'])
                download_file(r['from'], filename)
                url = os.path.join(settings.TILDA_MEDIA_IMAGES_URL, r['to'])
                page.html = page.html.replace(r['to'], url)
            page.save()

            for r in make_unique(result['css']):
                filename = os.path.join(settings.TILDA_MEDIA_CSS, r['to'])
                download_file(r['from'], filename)

            for r in make_unique(result['js']):
                filename = os.path.join(settings.TILDA_MEDIA_JS, r['to'])
                download_file(r['from'], filename)

            return True
    return False



# coding: utf-8
import os
from django.db import models
from django.conf import settings
from django.utils.translation import gettext_lazy as _


class TildaPage(models.Model):

    id = models.CharField(
        _(u'Page id'),
        max_length=50,
        primary_key=True,
        unique=True
    )

    title = models.CharField(
        _(u'Title'),
        max_length=100
    )

    html = models.TextField(
        _(u'HTML'),
        blank=True
    )

    images = models.TextField(
        _(u'Images'),
        blank=True
    )

    css = models.TextField(
        _(u'CSS'),
        blank=True
    )

    js = models.TextField(
        _(u'JS'),
        blank=True
    )

    synchronized = models.DateTimeField(
        _(u'Synchronized time'),
        blank=True,
        null=True
    )

    created = models.DateTimeField(
        _(u'Created'),
        auto_now_add=True
    )

    class Meta:
        ordering = ('title', )
        verbose_name = _(u'page')
        verbose_name_plural = _(u'Tilda Pages')

    def get_images_list(self):
        if self.images:
            return [
                os.path.join('/media/tilda/images', r['to'])
                for r in eval(self.images)
            ]
        return []

    def get_css_list(self):
        if self.css:
            return [
                os.path.join('/media/tilda/css', r['to'])
                for r in eval(self.css)
            ]
        return []

    def get_js_list(self):
        if self.js:
            return [
                os.path.join('/media/tilda/js', r['to'])
                for r in eval(self.js)
            ]
        return []

    def _path_images_list(self):
        if self.images:
            return [
                os.path.join(settings.TILDA_MEDIA_IMAGES, r['to'])
                for r in eval(self.images)
            ]
        return []

    def _path_css_list(self):
        if self.css:
            return [
                os.path.join(settings.TILDA_MEDIA_CSS, r['to'])
                for r in eval(self.css)
            ]
        return []

    def _path_js_list(self):
        if self.js:
            return [
                os.path.join(settings.TILDA_MEDIA_JS, r['to'])
                for r in eval(self.js)
            ]
        return []

    def __unicode__(self):
        return self.title

    def __str__(self):
        return self.title
